@@ -10,6 +10,7 @@ VISUALIZE = (("-m","--map"),{'action':'store_true','dest':'visualize','default':
 CONFIG = [
     ("npz",{'help':"route using NPZ graph",'args':[INDIRECT,PENALIZE,SOURCE,TARGET,OUT,VISUALIZE,GRAPH]}),
     ("nx",{'help':"route using NX graph",'args':[SOURCE,TARGET,OUT,VISUALIZE,GRAPH]}),
+    ("cnx",{'help':"route using CNX graph",'args':[SOURCE,TARGET,OUT,VISUALIZE,GRAPH]}),
     ("gt",{'help':"route using GT graph",'args':[INDIRECT,PENALIZE,SOURCE,TARGET,OUT,VISUALIZE,GRAPH]}),
     ("direct",{'help':"route directly",'args':[
         (("-s","--source"),{'type':int,'dest':'source_id','help':"set source power tower to SOURCE (default: random)",'metavar':'SOURCE','required':True}),
@@ -171,6 +172,65 @@ def route_nx(file_name,source_id=None,target_id=None,out_file=None,visualize=Fal
     start("Saving path to",out_file)
     save_path(path,out_file,visualize)
     end()
+    
+def astar_cnx(g,source_id=None,target_id=None):
+    from limic.util import locate_by_id, distance
+    #TODO handle locate_by_id correctly by constructing ids?
+    cs,node2closest,node2c = g
+    if isinstance(source_id,tuple) and isinstance(target_id,tuple):
+        source, target = source_id[0], target_id[0]
+    else:
+        nodes = list(node2closest.keys())
+        nodes.extend(node2c.keys())
+        ids = list(map(lambda x:x[0],nodes))
+        source, target = locate_by_id(ids,source_id,target_id)
+        source, target = nodes[source], nodes[target]
+    if source not in node2closest:
+        if target in node2closest or node2c[source] != node2c[target]:
+            cost = dist = distance(source,target)
+            path = None,[(0.,0.,False)+source,(float('inf'),dist,True)+target]
+            return path
+        path = astar_nx(c,(source,source_id),(target,target_id))
+        return path
+    paths = []
+    def extend(cp1,cp2):
+        cost = cp1[0]
+        path = cp1[1]
+        assert(path[-1][3:] == cp2[1][0][3:])
+        path.extend(map(lambda x:((x[0]+cost,)+x[1:]),cp2[1][1:]))
+        cost += cp2[0]
+        return cost, path
+    for ns,ps,cs in node2closest[source]:
+        for nt,pt,ct in node2closest[target]:
+            c = node2c[ns]
+            if c != node2c[nt]:
+                cost = dist = distance(source,target)
+                path = None,[(0.,0.,False)+source,(float('inf'),dist,True)+target]
+                return path
+            cp = astar_nx(ps,(source,None),(ns,None))
+            cpc = astar_nx(c,(ns,None),(nt,None))
+            for u,v in zip(cpc[1],cpc[1][1:]):
+                u,v = u[3:], v[3:]
+                cuv = c[u][v]
+                sg = cuv["path"]
+                cpuv = astar_nx(sg,(u,None),(v,None))
+                cp = extend(cp,cpuv)
+            cpt = astar_nx(pt,(nt,None),(target,None))
+            cp = extend(cp,cpt)
+            paths.append(cp)
+    return min(paths)
+
+def route_cnx(file_name,source_id=None,target_id=None,out_file=None,visualize=False):
+    from limic.util import start, end, load_pickled, save_path
+    start("Loading from",file_name)
+    g = load_pickled(file_name)
+    end()
+    start("Routing using condensed NX")
+    path = astar_cnx(g,source_id,target_id)
+    end()
+    start("Saving path to",out_file)
+    save_path(path,out_file,visualize)
+    end()
 
 def astar_gt(g,source_id=None,target_id=None):
     from graph_tool.search import astar_search, AStarVisitor, StopSearch
